/***************************************************************************
 *
 *  File:           fpga.ld
 *
 *  Description:    Linker Script
 *
 *  Date:           29 August 2001
 *
 *  Author:         A. Parker
 *
 *  Copyright:      Oxford Semiconductor Ltd, 2001
 *
 *  $Log: 940_fpga_rom_thumb.ld,v $
 *  Revision 1.1  2002/05/30 09:01:55  parker
 *  *** empty log message ***
 *
 *  Revision 1.1  2002/05/27 11:37:01  simmons
 *  linker script for 940 that uses SDRAM
 *
 *  Revision 1.1  2002/04/22 10:47:53  parker
 *  *** empty log message ***
 *
 *  Revision 1.3  2002/02/27 12:32:55  parker
 *  fixed alignment of ctors and dtors
 *
 *  Revision 1.2  2002/02/01 16:41:48  parker
 *  added explicit heap base section
 *
 *  Revision 1.1  2001/11/30 15:40:25  parker
 *  added thumb mode library link option
 *
 *  Revision 1.3  2001/11/19 16:47:12  parker
 *  added heap area, above static data
 *
 *  Revision 1.2  2001/10/10 08:42:06  parker
 *  stripped dos eols
 *
 *  Revision 1.1  2001/09/26 09:37:29  parker
 *  first rev
 *
 *
 ***************************************************************************/
/*Start up defines the first object file in the link process */
STARTUP(rominit.o)

/* the entry point */
ENTRY(_ENTRY)

/* specify general object files without including them in any section definition */ 
/* INPUT(crtbegin.o crtend.o) */

/* similar to input, except that the files are archive libraries */
/*GROUP(libgcc.a /tmp/build/newlib/arm-elf/newlib/libc.a)*/
GROUP(thumb/libgcc.a thumb/libc.a thumb/libstdc++.a)


/* specify the memory layout */ 
MEMORY
{
    rom   : ORIGIN = 0x0,        LENGTH = 0x30000
    ram   : ORIGIN = 0x30000,    LENGTH = 0x10000
    sdram : ORIGIN = 0x44000000, LENGTH = 0x800000
}


SECTIONS
{
  .init          : 
  { 
    KEEP (*(.init))
  } =0


    .text ALIGN (0x4)  : 
     { 
       *(.text*) 

 
       *(.gnu.warning) 
       *(.gnu.linkonce*) 
       *(.glue_7)  
       *(.glue_7t) 
       *(.rodata*) 
       *(.rel.got) 
 
     } >rom
  .fini      :
  {
    KEEP (*(.fini))
  } =0

  .ctors   : 
  {
    . = ALIGN(4);
     __CTOR_LIST__ = .; 
    /* gcc uses crtbegin.o to find the start of
       the constructors, so we make sure it is
       first.  Because this is a wildcard, it
       doesn't matter if the user does not
       actually link against crtbegin.o; the
       linker won't look for a file to match a
       wildcard.  The wildcard also means that it
       doesn't matter which directory crtbegin.o
       is in.  */
    /* KEEP (*crtbegin.o(.ctors)) */
    /* We don't want to include the .ctor section from
       from the crtend.o file until after the sorted ctors.
       The .ctor section from the crtend file contains the
       end of ctors marker and it must be last */
    /* KEEP (*(EXCLUDE_FILE (*crtend.o) .ctors)) */
    /* KEEP (*(SORT(.ctors.*))) */
    KEEP (*(.ctors))
     __CTOR_END__ = .;
    . = ALIGN(4);
  } > rom = 0
   .dtors         :
  {
    . = ALIGN(4);
    __DTOR_LIST__ = .; 

    /* KEEP (*crtbegin.o(.dtors)) */
    /* KEEP (*(EXCLUDE_FILE (*crtend.o ) .dtors)) */
    /* KEEP (*(SORT(.dtors.*))) */
    KEEP (*(.dtors))
     __DTOR_END__ = .;
    . = ALIGN(4);
  } > rom = 0
  PROVIDE (__etext = .);
  PROVIDE (_etext = .);
  PROVIDE (etext = .);

    _ramstart = . ;
    .mdata 0x30000  :
     AT ( ADDR(.dtors) + SIZEOF(.dtors) ) 
     {
      _data = .; 
      *(.gcc_except*) 
      *(.data*) 
      *(.got*) ; 
      . = ALIGN(4);
      _edata = .;
     } >ram

    .bss ALIGN (0x4) : 
     {  _bstart = .; 
        *(.bss) 
        *(COMMON) ;
        . = ALIGN(4);
        _bend = . ;
     } >ram

    .heap (0x44000000) : 
     AT (ADDR(.dtors) + SIZEOF(.dtors) + SIZEOF(.mdata))
     {
      _heapstart = .; 
      *(.heap_base) ; 
      *(.heap) ; 
      _eheap = .;
     } >sdram

    PROVIDE (end =.);
}


